#include <stdafx.h>
#include "BitmapRegion.h"
#include <assert.h>

using namespace System;
using namespace System::Windows;
using namespace System::Windows::Forms;


void BitmapRegion::Create(Control^ control,
		Drawing::Bitmap^ bitmap)
{
	assert(control != nullptr && bitmap != nullptr);
	control->Width = bitmap->Width;
	control->Height = bitmap->Height;

	Form ^frm = dynamic_cast<Form^>(control);
	if (frm != nullptr)
	{
		frm->Width += 15;
		frm->Height += 35;
		frm->FormBorderStyle = FormBorderStyle::None;

		Drawing::Drawing2D::GraphicsPath^ graphicsPath = CalculateControlGraphicsPath(bitmap);
		frm->Region = gcnew Drawing::Region(graphicsPath);
	}
}

Drawing::Drawing2D::GraphicsPath^ BitmapRegion::CalculateControlGraphicsPath(Drawing::Bitmap^ bitmap)
{
	Drawing::Drawing2D::GraphicsPath^ graphicsPath = gcnew Drawing::Drawing2D::GraphicsPath();

	// Use the top left pixel as our transparent color
	Drawing::Color colorTransparent = bitmap->GetPixel(0, 0);

	// This is to store the column value where an opaque pixel is first found.
	// This value will determine where we start scanning for trailing opaque pixels.
	int colOpaquePixel = 0;

	// Go through all rows (Y axis)
	for(int row = 0; row < bitmap->Height; row ++)
	{
		// Reset value
		colOpaquePixel = 0;

		// Go through all columns (X axis)
		for(int col = 0; col < bitmap->Width; col ++)
		{
			// If this is an opaque pixel, mark it and search for anymore trailing behind
			if(bitmap->GetPixel(col, row) != colorTransparent)
			{
				// Opaque pixel found, mark current position
				colOpaquePixel = col;

				// Create another variable to set the current pixel position
				int colNext = col;

				// Starting from current found opaque pixel, search for anymore opaque pixels 
				// trailing behind, until a transparent pixel is found or minimum width is reached
				for(colNext = colOpaquePixel; colNext < bitmap->Width; colNext ++)
				{	
					Drawing::Color pColor = bitmap->GetPixel(colNext, row);
					if(pColor == colorTransparent)
						break;
				}

				// Form a rectangle for line of opaque pixels found and add it to our graphics path
				graphicsPath->AddRectangle(System::Drawing::RectangleF(colOpaquePixel, row, colNext - colOpaquePixel, 1));

				// No need to scan the line of opaque pixels just found
				col = colNext;
			}
		}
	}

	// Return calculated graphics path
	return graphicsPath;
}
